# Copyright [2025] [IBM]
# Licensed under the Apache License, Version 2.0 (http://www.apache.org/licenses/LICENSE-2.0)
# See the LICENSE file in the project root for license information.

from pydantic import BaseModel, Field


class CreateAssetFromSqlQueryRequest(BaseModel):
    """Request model for creating asset from SQL query."""

    project_id: str = Field(..., description="The unique identifier of the project.")
    connection_id: str = Field(
        ...,
        description="The unique identifier of the connection.",
    )
    sql_query: str = Field(
        ..., description="The SQL query generated by the 'generate_sql' agent."
    )


class CreateAssetFromSqlQueryResponse(BaseModel):
    asset_url: str = Field(..., description="An URL of the newly created asset.")
